//
//  GiteaModels.swift
//  SwiftForge
//
//  Created by Atridad Lahiji on 2025-07-04.
//

import Foundation

// MARK: - User
struct User: Codable, Identifiable, Equatable {
    let id: Int
    let login: String
    let fullName: String?
    let email: String?
    let avatarUrl: String
    let htmlUrl: String
    let description: String?
    let website: String?
    let location: String?
    let active: Bool
    let isAdmin: Bool
    let followersCount: Int
    let followingCount: Int
    let starredReposCount: Int
    let created: Date
    let lastLogin: Date?

    enum CodingKeys: String, CodingKey {
        case id, login, email, active, website, location, created, description
        case fullName = "full_name"
        case avatarUrl = "avatar_url"
        case htmlUrl = "html_url"
        case isAdmin = "is_admin"
        case followersCount = "followers_count"
        case followingCount = "following_count"
        case starredReposCount = "starred_repos_count"
        case lastLogin = "last_login"
    }
}

// MARK: - Organization
struct Organization: Codable, Identifiable, Hashable, Equatable {
    let id: Int
    let username: String
    let name: String?
    let fullName: String?
    let email: String?
    let avatarUrl: String
    let description: String?
    let website: String?
    let location: String?
    let visibility: String

    enum CodingKeys: String, CodingKey {
        case id, username, name, email, description, website, location, visibility
        case fullName = "full_name"
        case avatarUrl = "avatar_url"
    }
}

// MARK: - Repository
struct Repository: Codable, Identifiable, Equatable {
    let id: Int
    let name: String
    let fullName: String
    let description: String?
    let htmlUrl: String
    let cloneUrl: String
    let sshUrl: String
    let owner: User
    let `private`: Bool
    let fork: Bool
    let template: Bool
    let empty: Bool
    let archived: Bool
    let mirror: Bool
    let size: Int
    let language: String?
    let languagesUrl: String
    let forksCount: Int
    let stargazersCount: Int
    let watchersCount: Int
    let openIssuesCount: Int
    let openPrCounter: Int
    let releaseCounter: Int
    let defaultBranch: String
    let createdAt: Date
    let updatedAt: Date
    let permissions: Permission?
    let hasIssues: Bool
    let hasWiki: Bool
    let hasPullRequests: Bool
    let hasProjects: Bool
    let hasReleases: Bool
    let hasPackages: Bool
    let hasActions: Bool
    let topics: [String]
    let avatarUrl: String?
    let internalTracker: InternalTracker?
    let externalTracker: ExternalTracker?
    let externalWiki: ExternalWiki?

    enum CodingKeys: String, CodingKey {
        case id, name, description, owner, fork, template, empty, archived, mirror, size
        case language, permissions, topics
        case fullName = "full_name"
        case htmlUrl = "html_url"
        case cloneUrl = "clone_url"
        case sshUrl = "ssh_url"
        case `private` = "private"
        case languagesUrl = "languages_url"
        case forksCount = "forks_count"
        case stargazersCount = "stars_count"
        case watchersCount = "watchers_count"
        case openIssuesCount = "open_issues_count"
        case openPrCounter = "open_pr_counter"
        case releaseCounter = "release_counter"
        case defaultBranch = "default_branch"
        case createdAt = "created_at"
        case updatedAt = "updated_at"
        case hasIssues = "has_issues"
        case hasWiki = "has_wiki"
        case hasPullRequests = "has_pull_requests"
        case hasProjects = "has_projects"
        case hasReleases = "has_releases"
        case hasPackages = "has_packages"
        case hasActions = "has_actions"
        case avatarUrl = "avatar_url"
        case internalTracker = "internal_tracker"
        case externalTracker = "external_tracker"
        case externalWiki = "external_wiki"
    }
}

// MARK: - Permission
struct Permission: Codable, Equatable {
    let admin: Bool
    let push: Bool
    let pull: Bool
}

// MARK: - Internal Tracker
struct InternalTracker: Codable, Equatable {
    let enableTimeTracker: Bool
    let allowOnlyContributorsToTrackTime: Bool
    let enableIssueDependencies: Bool

    enum CodingKeys: String, CodingKey {
        case enableTimeTracker = "enable_time_tracker"
        case allowOnlyContributorsToTrackTime = "allow_only_contributors_to_track_time"
        case enableIssueDependencies = "enable_issue_dependencies"
    }
}

// MARK: - External Tracker
struct ExternalTracker: Codable, Equatable {
    let externalTrackerUrl: String
    let externalTrackerFormat: String
    let externalTrackerStyle: String
    let externalTrackerRegexpPattern: String

    enum CodingKeys: String, CodingKey {
        case externalTrackerUrl = "external_tracker_url"
        case externalTrackerFormat = "external_tracker_format"
        case externalTrackerStyle = "external_tracker_style"
        case externalTrackerRegexpPattern = "external_tracker_regexp_pattern"
    }
}

// MARK: - External Wiki
struct ExternalWiki: Codable, Equatable {
    let externalWikiUrl: String

    enum CodingKeys: String, CodingKey {
        case externalWikiUrl = "external_wiki_url"
    }
}

// MARK: - Asset
struct Asset: Codable, Identifiable {
    let id: Int
    let name: String
    let size: Int
    let downloadCount: Int
    let createdAt: Date
    let browserDownloadUrl: String
    let uuid: String

    enum CodingKeys: String, CodingKey {
        case id, name, size, uuid
        case downloadCount = "download_count"
        case createdAt = "created_at"
        case browserDownloadUrl = "browser_download_url"
    }
}

// MARK: - Issue
// MARK: - Issue Repository (simplified for issue responses)
struct IssueRepository: Codable {
    let id: Int
    let name: String
    let owner: String
    let fullName: String

    enum CodingKeys: String, CodingKey {
        case id, name, owner
        case fullName = "full_name"
    }
}

struct Issue: Codable, Identifiable {
    let id: Int
    let number: Int
    let title: String
    let body: String?
    let user: User
    let labels: [Label]
    let milestone: Milestone?
    let assignee: User?
    let assignees: [User]?
    let state: IssueState
    let isLocked: Bool
    let comments: Int
    let createdAt: Date
    let updatedAt: Date
    let closedAt: Date?
    let dueDate: Date?
    let pullRequest: PullRequestMeta?
    let repository: IssueRepository?
    let htmlUrl: String
    let url: String
    let assets: [Asset]?
    let originalAuthor: String?
    let originalAuthorId: Int?
    let pinOrder: Int?
    let ref: String?

    enum CodingKeys: String, CodingKey {
        case id, number, title, body, user, labels, milestone, assignee, assignees
        case state, comments, repository, url, assets, ref
        case isLocked = "is_locked"
        case createdAt = "created_at"
        case updatedAt = "updated_at"
        case closedAt = "closed_at"
        case dueDate = "due_date"
        case pullRequest = "pull_request"
        case htmlUrl = "html_url"
        case originalAuthor = "original_author"
        case originalAuthorId = "original_author_id"
        case pinOrder = "pin_order"
    }

    init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)

        id = try container.decode(Int.self, forKey: .id)
        number = try container.decode(Int.self, forKey: .number)
        title = try container.decode(String.self, forKey: .title)
        body = try container.decodeIfPresent(String.self, forKey: .body)
        user = try container.decode(User.self, forKey: .user)
        labels = try container.decode([Label].self, forKey: .labels)
        milestone = try container.decodeIfPresent(Milestone.self, forKey: .milestone)
        assignee = try container.decodeIfPresent(User.self, forKey: .assignee)
        assignees = try container.decodeIfPresent([User].self, forKey: .assignees)

        // Handle state as string and convert to enum
        let stateString = try container.decode(String.self, forKey: .state)
        state = IssueState(rawValue: stateString) ?? .open
        isLocked = try container.decode(Bool.self, forKey: .isLocked)
        comments = try container.decode(Int.self, forKey: .comments)
        createdAt = try container.decode(Date.self, forKey: .createdAt)
        updatedAt = try container.decode(Date.self, forKey: .updatedAt)
        closedAt = try container.decodeIfPresent(Date.self, forKey: .closedAt)
        dueDate = try container.decodeIfPresent(Date.self, forKey: .dueDate)
        pullRequest = try container.decodeIfPresent(PullRequestMeta.self, forKey: .pullRequest)
        repository = try container.decodeIfPresent(IssueRepository.self, forKey: .repository)
        htmlUrl = try container.decode(String.self, forKey: .htmlUrl)
        url = try container.decode(String.self, forKey: .url)
        assets = try container.decodeIfPresent([Asset].self, forKey: .assets)
        originalAuthor = try container.decodeIfPresent(String.self, forKey: .originalAuthor)
        originalAuthorId = try container.decodeIfPresent(Int.self, forKey: .originalAuthorId)
        pinOrder = try container.decodeIfPresent(Int.self, forKey: .pinOrder)
        ref = try container.decodeIfPresent(String.self, forKey: .ref)
    }
}

// MARK: - Issue State
enum IssueState: String, Codable, CaseIterable {
    case open = "open"
    case closed = "closed"
    case all = "all"
}

// MARK: - Pull Request Meta
struct PullRequestMeta: Codable {
    let draft: Bool
    let htmlUrl: String
    let merged: Bool
    let mergedAt: Date?

    enum CodingKeys: String, CodingKey {
        case draft, merged
        case htmlUrl = "html_url"
        case mergedAt = "merged_at"
    }
}

// MARK: - Pull Request
struct PullRequest: Codable, Identifiable {
    let id: Int
    let number: Int
    let title: String
    let body: String?
    let user: User
    let labels: [Label]
    let milestone: Milestone?
    let assignee: User?
    let assignees: [User]
    let requestedReviewers: [User]
    let requestedReviewersTeams: [Team]
    let state: IssueState
    let isLocked: Bool
    let comments: Int
    let reviewComments: Int
    let draft: Bool
    let merged: Bool
    let mergeable: Bool?
    let mergedAt: Date?
    let mergedBy: User?
    let base: PRBranchInfo
    let head: PRBranchInfo
    let mergeBase: String
    let dueDate: Date?
    let createdAt: Date
    let updatedAt: Date
    let closedAt: Date?
    let htmlUrl: String
    let url: String
    let diffUrl: String
    let patchUrl: String

    enum CodingKeys: String, CodingKey {
        case id, number, title, body, user, labels, milestone, assignee, assignees
        case state, comments, draft, merged, mergeable, base, head, url
        case isLocked = "is_locked"
        case requestedReviewers = "requested_reviewers"
        case requestedReviewersTeams = "requested_reviewers_teams"
        case reviewComments = "review_comments"
        case mergedAt = "merged_at"
        case mergedBy = "merged_by"
        case mergeBase = "merge_base"
        case dueDate = "due_date"
        case createdAt = "created_at"
        case updatedAt = "updated_at"
        case closedAt = "closed_at"
        case htmlUrl = "html_url"
        case diffUrl = "diff_url"
        case patchUrl = "patch_url"
    }
}

// MARK: - PR Branch Info
struct PRBranchInfo: Codable {
    let label: String
    let ref: String
    let sha: String
    let repoId: Int
    let repo: Repository?

    enum CodingKeys: String, CodingKey {
        case label, ref, sha, repo
        case repoId = "repo_id"
    }
}

// MARK: - Team
struct Team: Codable, Identifiable {
    let id: Int
    let name: String
    let description: String?
    let organization: Organization?
    let permission: TeamPermission
    let includesAllRepositories: Bool
    let canCreateOrgRepo: Bool

    enum CodingKeys: String, CodingKey {
        case id, name, description, organization, permission
        case includesAllRepositories = "includes_all_repositories"
        case canCreateOrgRepo = "can_create_org_repo"
    }
}

// MARK: - Team Permission
enum TeamPermission: String, Codable {
    case none = "none"
    case read = "read"
    case write = "write"
    case admin = "admin"
    case owner = "owner"
}

// MARK: - Label
struct Label: Codable, Identifiable {
    let id: Int
    let name: String
    let color: String
    let description: String?
    let exclusive: Bool
    let isArchived: Bool
    let url: String

    enum CodingKeys: String, CodingKey {
        case id, name, color, description, exclusive, url
        case isArchived = "is_archived"
    }
}

// MARK: - Milestone
struct Milestone: Codable, Identifiable {
    let id: Int
    let title: String
    let description: String?
    let state: MilestoneState
    let openIssues: Int
    let closedIssues: Int
    let createdAt: Date
    let updatedAt: Date?
    let closedAt: Date?
    let dueOn: Date?

    enum CodingKeys: String, CodingKey {
        case id, title, description, state
        case openIssues = "open_issues"
        case closedIssues = "closed_issues"
        case createdAt = "created_at"
        case updatedAt = "updated_at"
        case closedAt = "closed_at"
        case dueOn = "due_on"
    }
}

// MARK: - Milestone State
enum MilestoneState: String, Codable {
    case open = "open"
    case closed = "closed"
}

// MARK: - Comment
struct Comment: Codable, Identifiable {
    let id: Int
    let body: String
    let user: User
    let createdAt: Date
    let updatedAt: Date
    let htmlUrl: String
    let issueUrl: String
    let pullRequestUrl: String?

    enum CodingKeys: String, CodingKey {
        case id, body, user
        case createdAt = "created_at"
        case updatedAt = "updated_at"
        case htmlUrl = "html_url"
        case issueUrl = "issue_url"
        case pullRequestUrl = "pull_request_url"
    }
}

// MARK: - Commit
struct Commit: Codable, Identifiable {
    let sha: String
    let url: String
    let htmlUrl: String
    let author: User?
    let committer: User?
    let commit: CommitInfo
    let parents: [CommitMeta]
    let files: [CommitFile]?
    let stats: CommitStats?
    let created: Date

    var id: String { sha }

    enum CodingKeys: String, CodingKey {
        case sha, url, author, committer, commit, parents, files, stats, created
        case htmlUrl = "html_url"
    }
}

// MARK: - Commit Info
struct CommitInfo: Codable {
    let url: String
    let author: CommitUser?
    let committer: CommitUser?
    let message: String
    let tree: GitObject
    let verification: CommitVerification?
}

// MARK: - Commit User
struct CommitUser: Codable {
    let name: String
    let email: String
    let date: Date
}

// MARK: - Git Object
struct GitObject: Codable {
    let sha: String
    let url: String
}

// MARK: - Commit Meta
struct CommitMeta: Codable {
    let sha: String
    let url: String
    let created: Date
}

// MARK: - Branch Commit User
struct BranchCommitUser: Codable {
    let name: String
    let email: String
    let username: String
}

// MARK: - Branch Commit (different structure than CommitMeta)
struct BranchCommit: Codable {
    let id: String
    let message: String
    let url: String
    let author: BranchCommitUser
    let committer: BranchCommitUser
    let verification: CommitVerification
    let timestamp: Date
}

// MARK: - Commit File
struct CommitFile: Codable {
    let filename: String
    let additions: Int?
    let deletions: Int?
    let changes: Int?
    let status: String
    let rawUrl: String?
    let blobUrl: String?
    let patchUrl: String?

    enum CodingKeys: String, CodingKey {
        case filename, additions, deletions, changes, status
        case rawUrl = "raw_url"
        case blobUrl = "blob_url"
        case patchUrl = "patch_url"
    }
}

// MARK: - Commit Stats
struct CommitStats: Codable {
    let total: Int
    let additions: Int
    let deletions: Int
}

// MARK: - Signer
struct Signer: Codable {
    let name: String
    let email: String
    let username: String
}

// MARK: - Commit Verification
struct CommitVerification: Codable {
    let verified: Bool
    let reason: String
    let signature: String
    let payload: String
    let signer: Signer?
}

// MARK: - Branch
struct Branch: Codable, Identifiable {
    let name: String
    let commit: BranchCommit
    let protected: Bool
    let requiredApprovals: Int
    let enableStatusCheck: Bool
    let statusCheckContexts: [String]
    let userCanPush: Bool
    let userCanMerge: Bool
    let effectiveBranchProtectionName: String

    var id: String { name }

    enum CodingKeys: String, CodingKey {
        case name, commit, protected
        case requiredApprovals = "required_approvals"
        case enableStatusCheck = "enable_status_check"
        case statusCheckContexts = "status_check_contexts"
        case userCanPush = "user_can_push"
        case userCanMerge = "user_can_merge"
        case effectiveBranchProtectionName = "effective_branch_protection_name"
    }
}

// MARK: - Release
struct Release: Codable, Identifiable {
    let id: Int
    let name: String
    let body: String?
    let tagName: String
    let targetCommitish: String
    let draft: Bool
    let prerelease: Bool
    let createdAt: Date
    let publishedAt: Date?
    let author: User
    let url: String
    let htmlUrl: String
    let assetsUrl: String
    let assets: [ReleaseAsset]
    let tarballUrl: String
    let zipballUrl: String

    enum CodingKeys: String, CodingKey {
        case id, name, body, draft, prerelease, author, url, assets
        case tagName = "tag_name"
        case targetCommitish = "target_commitish"
        case createdAt = "created_at"
        case publishedAt = "published_at"
        case htmlUrl = "html_url"
        case assetsUrl = "assets_url"
        case tarballUrl = "tarball_url"
        case zipballUrl = "zipball_url"
    }
}

// MARK: - Release Asset
struct ReleaseAsset: Codable, Identifiable {
    let id: Int
    let name: String
    let size: Int
    let downloadCount: Int
    let createdAt: Date
    let uuid: String
    let browserDownloadUrl: String

    enum CodingKeys: String, CodingKey {
        case id, name, size, uuid
        case downloadCount = "download_count"
        case createdAt = "created_at"
        case browserDownloadUrl = "browser_download_url"
    }
}

// MARK: - Notification
struct Notification: Codable, Identifiable {
    let id: Int
    let unread: Bool
    let pinned: Bool
    let subject: NotificationSubject
    let repository: Repository
    let url: String
    let updatedAt: Date

    enum CodingKeys: String, CodingKey {
        case id, unread, pinned, subject, repository, url
        case updatedAt = "updated_at"
    }
}

// MARK: - Notification Subject
struct NotificationSubject: Codable {
    let title: String
    let url: String
    let type: NotificationType
    let state: IssueState?
    let htmlUrl: String
    let latestCommentUrl: String?
    let latestCommentHtmlUrl: String?

    enum CodingKeys: String, CodingKey {
        case title, url, type, state
        case htmlUrl = "html_url"
        case latestCommentUrl = "latest_comment_url"
        case latestCommentHtmlUrl = "latest_comment_html_url"
    }
}

// MARK: - Notification Type
enum NotificationType: String, Codable {
    case issue = "Issue"
    case pullRequest = "PullRequest"
    case commit = "Commit"
    case repository = "Repository"
}

// MARK: - API Response Wrappers
struct APIResponse<T: Codable>: Codable {
    let data: T
    let ok: Bool
}

struct PaginatedResponse<T: Codable>: Codable {
    let data: [T]
    let totalCount: Int
    let hasMore: Bool

    enum CodingKeys: String, CodingKey {
        case data
        case totalCount = "total_count"
        case hasMore = "has_more"
    }
}

// MARK: - Server Version
struct ServerVersion: Codable {
    let version: String
}

// MARK: - Search Results
struct SearchResults<T: Codable>: Codable {
    let data: [T]
    let ok: Bool
}

// MARK: - Authentication Token
struct AccessToken: Codable, Identifiable {
    let id: Int
    let name: String
    let sha1: String
    let tokenLastEight: String
    let scopes: [String]

    enum CodingKeys: String, CodingKey {
        case id, name, sha1, scopes
        case tokenLastEight = "token_last_eight"
    }
}
